/* ========================================================================== */
/*                                                                            */
/*   Mini-monitor routines for ATtiny2313 and comparables                     */
/*   (c) August 2010 Kees Moerman                                             */
/*   Description:                                                             */
/*   A mini (serial port driven, interrupt based) monitor program             */
/*   capable of reading/writing RAM memory locations and I/O ports            */
/*                                                                            */
/* 	Commands:                                                                 */
/*	- a <byte> sets address to <byte> (out: 'a=')                             */
/*	- r reads byte (out: value)                                               */
/*	- w <byte> writes byte to address (out: 'w!')                             */
/*	- i increment address (out: <value>)                                      */
/*	- space: just echoed                                                      */
/*  See MiniMon.exe on Windows for more user-friendly interface               */
/*	Reminder: Memory map 00..1F register file, 20..5F: I/O ports, 60..DF: RAM */
/*                                                                            */
/*	User callable function: void MiniMonInitUART (int baudrate_factor)        */
/*	Code size 0c + (E8 - 66) = 0x8E (142, 7%) bytes; 8 bytes stack, 2 RAM     */
/*                                                                            */
/* ========================================================================== */

#ifdef MINIMON
#else

#define MINIMON 1

// Pins and Functions
// ------------------
// PD0	Pin 2 	= RS232 RX (TTL, inverted) from RFID reader SEROUT, through an inverter
// PD1	Pin 3 	= RS232 TX (TTL, inverted) to serial LCD, through an inverter

// #include <avr/io.h>
// #include <avr/interrupt.h>


/* Initialize UART at 8n1 */
void MiniMonInitUART (int baudrate_factor)
{
	/* Set the baud rate */
	UBRRL = baudrate_factor;
	/* Enable UART receiver and transmitter, receive interrupt enable*/
	UCSRB = (1 << RXEN) | (1 << TXEN) | (1 << RXCIE);
	/* set to 8 data bits, no parity, 1 stop bit */
	UCSRC = (1 << UCSZ1) | (1 << UCSZ0);
}

uint8_t minimon_address;
uint8_t minimon_mode;

// Actual interrupt service routine running monitor program
ISR(USART_RX_vect)
{
    uint8_t char_in;
	char_in = UDR;
	if(minimon_mode == 0)
	{
		switch(char_in)
		{
			case ' ':	{ UDR = char_in; break; }				// nop
			case 'r':	{ UDR = *(uint8_t *)(uint16_t)minimon_address; break; }
			case 'i':	{ UDR = ++minimon_address; break; }	// increment
			default: 	{ minimon_mode = char_in; UDR = char_in; break; }
		}
	}
	else
	{
		switch(minimon_mode)
		{
			case 'a': { minimon_address = char_in; UDR = '='; break; }
			case 'w': { *(uint8_t *)(uint16_t)minimon_address = char_in; UDR = '!'; break; }
			default:  { UDR = '?'; break; }
		}
		minimon_mode = 0;
	}
}

#endif

// end of file
